@@ -4,8 +4,8 @@
 #  --> https://CRAN.R-project.org/package=spam              #
 #  --> https://CRAN.R-project.org/package=spam64            #
 #  --> https://git.math.uzh.ch/reinhard.furrer/spam         #
-# by Reinhard Furrer [aut, cre], Florian Gerber [ctb],      #
-#    Roman Flury [ctb], Daniel Gerber [ctb],                #
+# by Reinhard Furrer [aut, cre], Florian Gerber [aut],      #
+#    Roman Flury [aut], Daniel Gerber [ctb],                #
 #    Kaspar Moesinger [ctb]                                 #
 # HEADER END ################################################
 
@@ -13,8 +13,25 @@
 
 # Draw from a multivariate normal:
 # (Algorithm 2.3 from Rue and Held, 2005)
-rmvnorm.spam <- function(n, mu = rep.int(0, dim(Sigma)[1]), Sigma, Rstruct = NULL, ...) {
+rmvnorm <- function(n, mu = rep.int(0, dim(Sigma)[1]), Sigma, ..., mean, sigma) {
+
+  if (!missing(mean))   mu <- mean
+  if (!missing(sigma))  Sigma <- sigma
+
+  if (is( Sigma, "spam")) return( rmvnorm.spam( n, mu, Sigma, ...))
+
+  R <- chol( Sigma, ...)
+  N <- dim( Sigma)[1]
+
+  return(sweep( as.matrix( ( array(rnorm(n*N),c(n,N)) %*% R)), 2, mu, "+"))
+}
+
+rmvnorm.spam <- function(n, mu = rep.int(0, dim(Sigma)[1]), Sigma, Rstruct = NULL, ..., mean, sigma) {
   # taken from ?chol.spam
+  if (!missing(mean))   mu <- mean
+  if (!missing(sigma))  Sigma <- sigma
+
+  if (!is(Sigma, "spam"))  return(  rmvnorm(n, mu, Sigma, ...))  # just in case!
 
   if (is(Rstruct,"spam.chol.NgPeyton"))
     cholS <- update.spam.chol.NgPeyton( Rstruct, Sigma, ...)
@@ -32,6 +49,9 @@ rmvnorm.spam <- function(n, mu = rep.int(0, dim(Sigma)[1]), Sigma, Rstruct = NUL
   return(sweep(retval, 2, mu, "+"))
 }
 
+
+
+
 # Draw from a multivariate normal given a precision matrix:
 # (Algorithm 2.4 from Rue and Held, 2005)
 rmvnorm.prec <- function(n, mu = rep.int(0, dim(Q)[1]), Q, Rstruct = NULL, ...) {
@@ -145,3 +165,38 @@ rmvnorm.canonical.const <- function (n, b, Q, Rstruct = NULL,
 
 
 }
+
+
+
+### conditional simulation
+rmvnorm.conditional <- function(n, y, mu = rep.int(0, dim(SigmaXX)[1]+dim(SigmaYY)[1]),
+                                   SigmaXX, SigmaYY, SigmaXY,
+                                   noise,
+                                   RstructYY = NULL, ...) {
+
+  if (!is(SigmaXX, "spam"))  stop("Implemented only for 'spam' covariance matrices")
+
+  nx <- dim(SigmaXX)[1]
+  if (missing(SigmaYY)) {
+    if (missing(noise)) stop("Either 'noise' or 'SigmaYY' are required")
+    SigmaYY <- SigmaXX+diag.spam(noise, nx, nx)
+  } else if (!missing(noise)) stop("Either 'noise' or 'SigmaYY' are required, not both")
+
+  ny <- dim(SigmaYY)[1]
+  if (length(y)!=ny)  stop("Length of 'y' does not match dimension of 'SigmaYY'")
+
+  if (missing(SigmaXY)) {  # save one transpose
+    if (nx!=ny) stop("'SigmaXY' needs to be specified")
+    Sigma <- rbind.spam(cbind.spam(SigmaXX, SigmaXX), cbind(SigmaXX, SigmaYY))
+    SigmaXY <- SigmaXX   # alternative would be to have another if below...
+  } else {
+    if (!all(dim(SigmaXY) == c(nx,ny))) stop("Wrong dimension for 'SigmaXY'")
+    Sigma <- rbind.spam(cbind.spam(SigmaXX, SigmaXY), cbind(t(SigmaXY), SigmaYY))
+  }
+  #  Rstruct <- chol.spam(Sigma)
+  rsample <- t(rmvnorm.spam(n = n, mu = mu, Sigma = Sigma)) #, Rstruct = Rstruct)
+
+  t(rsample[1:nx,] + SigmaXY %*%
+      solve.spam(a = SigmaYY, b = y-rsample[(nx+1):(nx+ny),],
+                 Rstruct = RstructYY))
+}
